@@ -31,41 +31,26 @@
 # define inline __inline
 #endif
 
-#undef HAVE_FUTIMES
-#undef HAVE_KQUEUE
-#undef HAVE_PORTS_FS
-
-#if __linux__
-# include "linux/syscalls.h"
-# define HAVE_FUTIMES 1 /* emulated with utimesat() */
+#if defined(__linux__)
+# include "linux-syscalls.h"
 #endif /* __linux__ */
 
 #if defined(__sun)
 # include <sys/port.h>
 # include <port.h>
-# ifdef PORT_SOURCE_FILE
-#  define HAVE_PORTS_FS 1
-# endif
-# define HAVE_FUTIMES 1
 # define futimes(fd, tv) futimesat(fd, (void*)0, tv)
 #endif /* __sun */
 
-#if defined(__APPLE__) || defined(__FreeBSD__) || defined(__sun)
-# define HAVE_FUTIMES 1
-#endif
-
-/* FIXME exact copy of the #ifdef guard in uv-unix.h */
-#if defined(__APPLE__)  \
-  || defined(__FreeBSD__) \
-  || defined(__OpenBSD__) \
-  || defined(__NetBSD__)
-# define HAVE_KQUEUE 1
-#endif
-
-#if defined(__APPLE__) && !defined(TARGET_OS_IPHONE)
+#if defined(__APPLE__) && !TARGET_OS_IPHONE
 # include <CoreServices/CoreServices.h>
 #endif
 
+#define STATIC_ASSERT(expr)                                                   \
+  void uv__static_assert(int static_assert_failed[0 - !(expr)])
+
+#define ACCESS_ONCE(type, var)                                                \
+  (*(volatile type*) &(var))
+
 #define UNREACHABLE()                                                         \
   do {                                                                        \
     assert(0 && "unreachable code");                                          \
@@ -81,9 +66,35 @@
   }                                                                           \
   while (0)
 
-#define UV__IO_READ  EV_READ
-#define UV__IO_WRITE EV_WRITE
-#define UV__IO_ERROR EV_ERROR
+#if defined(__linux__)
+# define UV__POLLIN   UV__EPOLLIN
+# define UV__POLLOUT  UV__EPOLLOUT
+# define UV__POLLERR  UV__EPOLLERR
+# define UV__POLLHUP  UV__EPOLLHUP
+#endif
+
+#if defined(__sun)
+# define UV__POLLIN   POLLIN
+# define UV__POLLOUT  POLLOUT
+# define UV__POLLERR  POLLERR
+# define UV__POLLHUP  POLLHUP
+#endif
+
+#ifndef UV__POLLIN
+# define UV__POLLIN   1
+#endif
+
+#ifndef UV__POLLOUT
+# define UV__POLLOUT  2
+#endif
+
+#ifndef UV__POLLERR
+# define UV__POLLERR  4
+#endif
+
+#ifndef UV__POLLHUP
+# define UV__POLLHUP  8
+#endif
 
 /* handle flags */
 enum {
@@ -100,29 +111,26 @@ enum {
   UV_TCP_SINGLE_ACCEPT = 0x400  /* Only accept() when idle. */
 };
 
-__attribute__((unused))
-__attribute__((always_inline))
-static void uv__req_init(uv_loop_t* loop, uv_req_t* req, uv_req_type type) {
-  req->type = type;
-  uv__req_register(loop, req);
-}
-#define uv__req_init(loop, req, type) \
-  uv__req_init((loop), (uv_req_t*)(req), (type))
-
 /* core */
-void uv__handle_init(uv_loop_t* loop, uv_handle_t* handle, uv_handle_type type);
 int uv__nonblock(int fd, int set);
 int uv__cloexec(int fd, int set);
 int uv__socket(int domain, int type, int protocol);
 int uv__dup(int fd);
-int uv_async_stop(uv_async_t* handle);
+void uv__make_close_pending(uv_handle_t* handle);
 
-void uv__io_init(uv__io_t* handle, uv__io_cb cb, int fd, int events);
-void uv__io_set(uv__io_t* handle, uv__io_cb cb, int fd, int events);
-void uv__io_start(uv_loop_t* loop, uv__io_t* handle);
-void uv__io_stop(uv_loop_t* loop, uv__io_t* handle);
-void uv__io_feed(uv_loop_t* loop, uv__io_t* handle, int event);
-int uv__io_active(uv__io_t* handle);
+void uv__io_init(uv__io_t* w, uv__io_cb cb, int fd);
+void uv__io_start(uv_loop_t* loop, uv__io_t* w, unsigned int events);
+void uv__io_stop(uv_loop_t* loop, uv__io_t* w, unsigned int events);
+void uv__io_close(uv_loop_t* loop, uv__io_t* w);
+void uv__io_feed(uv_loop_t* loop, uv__io_t* w);
+int uv__io_active(const uv__io_t* w, unsigned int events);
+void uv__io_poll(uv_loop_t* loop, int timeout); /* in milliseconds or -1 */
+
+/* async */
+void uv__async_send(struct uv__async* wa);
+void uv__async_init(struct uv__async* wa);
+int uv__async_start(uv_loop_t* loop, struct uv__async* wa, uv__async_cb cb);
+void uv__async_stop(uv_loop_t* loop, struct uv__async* wa);
 
 /* loop */
 int uv__loop_init(uv_loop_t* loop, int default_loop);
@@ -140,33 +148,39 @@ void uv__stream_init(uv_loop_t* loop, uv_stream_t* stream,
     uv_handle_type type);
 int uv__stream_open(uv_stream_t*, int fd, int flags);
 void uv__stream_destroy(uv_stream_t* stream);
-void uv__server_io(uv_loop_t* loop, uv__io_t* watcher, int events);
+#if defined(__APPLE__)
+int uv__stream_try_select(uv_stream_t* stream, int* fd);
+#endif /* defined(__APPLE__) */
+void uv__server_io(uv_loop_t* loop, uv__io_t* w, unsigned int events);
 int uv__accept(int sockfd);
 
 /* tcp */
 int uv_tcp_listen(uv_tcp_t* tcp, int backlog, uv_connection_cb cb);
-int uv__tcp_nodelay(uv_tcp_t* handle, int enable);
-int uv__tcp_keepalive(uv_tcp_t* handle, int enable, unsigned int delay);
+int uv__tcp_nodelay(int fd, int on);
+int uv__tcp_keepalive(int fd, int on, unsigned int delay);
 
 /* pipe */
 int uv_pipe_listen(uv_pipe_t* handle, int backlog, uv_connection_cb cb);
 
 /* timer */
 void uv__run_timers(uv_loop_t* loop);
-unsigned int uv__next_timeout(uv_loop_t* loop);
+int uv__next_timeout(const uv_loop_t* loop);
 
 /* signal */
 void uv__signal_close(uv_signal_t* handle);
-void uv__signal_unregister(uv_loop_t* loop);
+void uv__signal_global_once_init(void);
+void uv__signal_loop_cleanup(uv_loop_t* loop);
 
 /* thread pool */
 void uv__work_submit(uv_loop_t* loop,
                      struct uv__work *w,
                      void (*work)(struct uv__work *w),
-                     void (*done)(struct uv__work *w));
+                     void (*done)(struct uv__work *w, int status));
 void uv__work_done(uv_async_t* handle, int status);
 
 /* platform specific */
+uint64_t uv__hrtime(void);
+int uv__kqueue_init(uv_loop_t* loop);
 int uv__platform_loop_init(uv_loop_t* loop, int default_loop);
 void uv__platform_loop_delete(uv_loop_t* loop);
 
@@ -185,6 +199,13 @@ void uv__timer_close(uv_timer_t* handle);
 void uv__udp_close(uv_udp_t* handle);
 void uv__udp_finish_close(uv_udp_t* handle);
 
+#if defined(__APPLE__)
+int uv___stream_fd(uv_stream_t* handle);
+#define uv__stream_fd(handle) (uv___stream_fd((uv_stream_t*) (handle)))
+#else
+#define uv__stream_fd(handle) ((handle)->io_watcher.fd)
+#endif /* defined(__APPLE__) */
+
 #ifdef UV__O_NONBLOCK
 # define UV__F_NONBLOCK UV__O_NONBLOCK
 #else
@@ -222,4 +243,17 @@ static const int kFSEventStreamEventFlagItemIsSymlink = 0x00040000;
 
 #endif /* defined(__APPLE__) */
 
+__attribute__((unused))
+static void uv__req_init(uv_loop_t* loop, uv_req_t* req, uv_req_type type) {
+  req->type = type;
+  uv__req_register(loop, req);
+}
+#define uv__req_init(loop, req, type) \
+  uv__req_init((loop), (uv_req_t*)(req), (type))
+
+__attribute__((unused))
+static void uv__update_time(uv_loop_t* loop) {
+  loop->time = uv__hrtime() / 1000000;
+}
+
 #endif /* UV_UNIX_INTERNAL_H_ */
