@@ -104,16 +104,16 @@ static char buf[32];
 static char test_buf[] = "test-buffer\n";
 
 
-void check_permission(const char* filename, int mode) {
+static void check_permission(const char* filename, int mode) {
   int r;
   uv_fs_t req;
-  struct stat* s;
+  uv_statbuf_t* s;
 
   r = uv_fs_stat(uv_default_loop(), &req, filename, NULL);
   ASSERT(r == 0);
   ASSERT(req.result == 0);
 
-  s = req.ptr;
+  s = &req.statbuf;
 #ifdef _WIN32
   /*
    * On Windows, chmod can only modify S_IWUSR (_S_IWRITE) bit,
@@ -477,11 +477,12 @@ TEST_IMPL(fs_file_noent) {
   ASSERT(r == 0);
 
   ASSERT(open_cb_count == 0);
-  uv_run(loop);
+  uv_run(loop, UV_RUN_DEFAULT);
   ASSERT(open_cb_count == 1);
 
   /* TODO add EACCES test */
 
+  MAKE_VALGRIND_HAPPY();
   return 0;
 }
 
@@ -505,9 +506,10 @@ TEST_IMPL(fs_file_nametoolong) {
   ASSERT(r == 0);
 
   ASSERT(open_cb_count == 0);
-  uv_run(loop);
+  uv_run(loop, UV_RUN_DEFAULT);
   ASSERT(open_cb_count == 1);
 
+  MAKE_VALGRIND_HAPPY();
   return 0;
 }
 
@@ -531,16 +533,17 @@ TEST_IMPL(fs_file_loop) {
   ASSERT(r == 0);
 
   ASSERT(open_cb_count == 0);
-  uv_run(loop);
+  uv_run(loop, UV_RUN_DEFAULT);
   ASSERT(open_cb_count == 1);
 
   unlink("test_symlink");
 
+  MAKE_VALGRIND_HAPPY();
   return 0;
 }
 
 static void check_utime(const char* path, double atime, double mtime) {
-  struct stat* s;
+  uv_statbuf_t* s;
   uv_fs_t req;
   int r;
 
@@ -548,7 +551,7 @@ static void check_utime(const char* path, double atime, double mtime) {
   ASSERT(r == 0);
 
   ASSERT(req.result == 0);
-  s = req.ptr;
+  s = &req.statbuf;
 
 #if defined(_WIN32) || defined(_AIX)
   ASSERT(s->st_atime == atime);
@@ -607,7 +610,7 @@ TEST_IMPL(fs_file_async) {
   r = uv_fs_open(loop, &open_req1, "test_file", O_WRONLY | O_CREAT,
       S_IREAD | S_IWRITE, create_cb);
   ASSERT(r == 0);
-  uv_run(loop);
+  uv_run(loop, UV_RUN_DEFAULT);
 
   ASSERT(create_cb_count == 1);
   ASSERT(write_cb_count == 1);
@@ -618,7 +621,7 @@ TEST_IMPL(fs_file_async) {
   r = uv_fs_rename(loop, &rename_req, "test_file", "test_file2", rename_cb);
   ASSERT(r == 0);
 
-  uv_run(loop);
+  uv_run(loop, UV_RUN_DEFAULT);
   ASSERT(create_cb_count == 1);
   ASSERT(write_cb_count == 1);
   ASSERT(close_cb_count == 1);
@@ -627,7 +630,7 @@ TEST_IMPL(fs_file_async) {
   r = uv_fs_open(loop, &open_req1, "test_file2", O_RDWR, 0, open_cb);
   ASSERT(r == 0);
 
-  uv_run(loop);
+  uv_run(loop, UV_RUN_DEFAULT);
   ASSERT(open_cb_count == 1);
   ASSERT(read_cb_count == 1);
   ASSERT(close_cb_count == 2);
@@ -639,7 +642,7 @@ TEST_IMPL(fs_file_async) {
   r = uv_fs_open(loop, &open_req1, "test_file2", O_RDONLY, 0, open_cb);
   ASSERT(r == 0);
 
-  uv_run(loop);
+  uv_run(loop, UV_RUN_DEFAULT);
   ASSERT(open_cb_count == 2);
   ASSERT(read_cb_count == 2);
   ASSERT(close_cb_count == 3);
@@ -653,6 +656,7 @@ TEST_IMPL(fs_file_async) {
   unlink("test_file");
   unlink("test_file2");
 
+  MAKE_VALGRIND_HAPPY();
   return 0;
 }
 
@@ -737,6 +741,7 @@ TEST_IMPL(fs_file_sync) {
   unlink("test_file");
   unlink("test_file2");
 
+  MAKE_VALGRIND_HAPPY();
   return 0;
 }
 
@@ -754,7 +759,7 @@ TEST_IMPL(fs_async_dir) {
   r = uv_fs_mkdir(loop, &mkdir_req, "test_dir", 0755, mkdir_cb);
   ASSERT(r == 0);
 
-  uv_run(loop);
+  uv_run(loop, UV_RUN_DEFAULT);
   ASSERT(mkdir_cb_count == 1);
 
   /* Create 2 files synchronously. */
@@ -777,7 +782,7 @@ TEST_IMPL(fs_async_dir) {
   r = uv_fs_readdir(loop, &readdir_req, "test_dir", 0, readdir_cb);
   ASSERT(r == 0);
 
-  uv_run(loop);
+  uv_run(loop, UV_RUN_DEFAULT);
   ASSERT(readdir_cb_count == 1);
 
   /* sync uv_fs_readdir */
@@ -792,35 +797,35 @@ TEST_IMPL(fs_async_dir) {
 
   r = uv_fs_stat(loop, &stat_req, "test_dir", stat_cb);
   ASSERT(r == 0);
-  uv_run(loop);
+  uv_run(loop, UV_RUN_DEFAULT);
 
   r = uv_fs_stat(loop, &stat_req, "test_dir/", stat_cb);
   ASSERT(r == 0);
-  uv_run(loop);
+  uv_run(loop, UV_RUN_DEFAULT);
 
   r = uv_fs_lstat(loop, &stat_req, "test_dir", stat_cb);
   ASSERT(r == 0);
-  uv_run(loop);
+  uv_run(loop, UV_RUN_DEFAULT);
 
   r = uv_fs_lstat(loop, &stat_req, "test_dir/", stat_cb);
   ASSERT(r == 0);
-  uv_run(loop);
+  uv_run(loop, UV_RUN_DEFAULT);
 
   ASSERT(stat_cb_count == 4);
 
   r = uv_fs_unlink(loop, &unlink_req, "test_dir/file1", unlink_cb);
   ASSERT(r == 0);
-  uv_run(loop);
+  uv_run(loop, UV_RUN_DEFAULT);
   ASSERT(unlink_cb_count == 1);
 
   r = uv_fs_unlink(loop, &unlink_req, "test_dir/file2", unlink_cb);
   ASSERT(r == 0);
-  uv_run(loop);
+  uv_run(loop, UV_RUN_DEFAULT);
   ASSERT(unlink_cb_count == 2);
 
   r = uv_fs_rmdir(loop, &rmdir_req, "test_dir", rmdir_cb);
   ASSERT(r == 0);
-  uv_run(loop);
+  uv_run(loop, UV_RUN_DEFAULT);
   ASSERT(rmdir_cb_count == 1);
 
   /* Cleanup */
@@ -828,6 +833,7 @@ TEST_IMPL(fs_async_dir) {
   unlink("test_dir/file2");
   rmdir("test_dir");
 
+  MAKE_VALGRIND_HAPPY();
   return 0;
 }
 
@@ -872,7 +878,7 @@ TEST_IMPL(fs_async_sendfile) {
   r = uv_fs_sendfile(loop, &sendfile_req, open_req2.result, open_req1.result,
       0, 131072, sendfile_cb);
   ASSERT(r == 0);
-  uv_run(loop);
+  uv_run(loop, UV_RUN_DEFAULT);
 
   ASSERT(sendfile_cb_count == 1);
 
@@ -891,6 +897,7 @@ TEST_IMPL(fs_async_sendfile) {
   unlink("test_file");
   unlink("test_file2");
 
+  MAKE_VALGRIND_HAPPY();
   return 0;
 }
 
@@ -928,7 +935,7 @@ TEST_IMPL(fs_fstat) {
   /* Now do the uv_fs_fstat call asynchronously */
   r = uv_fs_fstat(loop, &req, file, fstat_cb);
   ASSERT(r == 0);
-  uv_run(loop);
+  uv_run(loop, UV_RUN_DEFAULT);
   ASSERT(fstat_cb_count == 1);
 
 
@@ -941,11 +948,12 @@ TEST_IMPL(fs_fstat) {
    * Run the loop just to check we don't have make any extraneous uv_ref()
    * calls. This should drop out immediately.
    */
-  uv_run(loop);
+  uv_run(loop, UV_RUN_DEFAULT);
 
   /* Cleanup. */
   unlink("test_file");
 
+  MAKE_VALGRIND_HAPPY();
   return 0;
 }
 
@@ -1006,7 +1014,7 @@ TEST_IMPL(fs_chmod) {
   }
   r = uv_fs_chmod(loop, &req, "test_file", 0200, chmod_cb);
   ASSERT(r == 0);
-  uv_run(loop);
+  uv_run(loop, UV_RUN_DEFAULT);
   ASSERT(chmod_cb_count == 1);
   chmod_cb_count = 0; /* reset for the next test */
 #endif
@@ -1018,7 +1026,7 @@ TEST_IMPL(fs_chmod) {
   }
   r = uv_fs_chmod(loop, &req, "test_file", 0400, chmod_cb);
   ASSERT(r == 0);
-  uv_run(loop);
+  uv_run(loop, UV_RUN_DEFAULT);
   ASSERT(chmod_cb_count == 1);
 
   /* async fchmod */
@@ -1028,7 +1036,7 @@ TEST_IMPL(fs_chmod) {
   }
   r = uv_fs_fchmod(loop, &req, file, 0600, fchmod_cb);
   ASSERT(r == 0);
-  uv_run(loop);
+  uv_run(loop, UV_RUN_DEFAULT);
   ASSERT(fchmod_cb_count == 1);
 
   close(file);
@@ -1037,11 +1045,12 @@ TEST_IMPL(fs_chmod) {
    * Run the loop just to check we don't have make any extraneous uv_ref()
    * calls. This should drop out immediately.
    */
-  uv_run(loop);
+  uv_run(loop, UV_RUN_DEFAULT);
 
   /* Cleanup. */
   unlink("test_file");
 
+  MAKE_VALGRIND_HAPPY();
   return 0;
 }
 
@@ -1078,19 +1087,19 @@ TEST_IMPL(fs_chown) {
   /* async chown */
   r = uv_fs_chown(loop, &req, "test_file", -1, -1, chown_cb);
   ASSERT(r == 0);
-  uv_run(loop);
+  uv_run(loop, UV_RUN_DEFAULT);
   ASSERT(chown_cb_count == 1);
 
   /* chown to root (fail) */
   chown_cb_count = 0;
   r = uv_fs_chown(loop, &req, "test_file", 0, 0, chown_root_cb);
-  uv_run(loop);
+  uv_run(loop, UV_RUN_DEFAULT);
   ASSERT(chown_cb_count == 1);
 
   /* async fchown */
   r = uv_fs_fchown(loop, &req, file, -1, -1, fchown_cb);
   ASSERT(r == 0);
-  uv_run(loop);
+  uv_run(loop, UV_RUN_DEFAULT);
   ASSERT(fchown_cb_count == 1);
 
   close(file);
@@ -1099,11 +1108,12 @@ TEST_IMPL(fs_chown) {
    * Run the loop just to check we don't have make any extraneous uv_ref()
    * calls. This should drop out immediately.
    */
-  uv_run(loop);
+  uv_run(loop, UV_RUN_DEFAULT);
 
   /* Cleanup. */
   unlink("test_file");
 
+  MAKE_VALGRIND_HAPPY();
   return 0;
 }
 
@@ -1158,7 +1168,7 @@ TEST_IMPL(fs_link) {
   /* async link */
   r = uv_fs_link(loop, &req, "test_file", "test_file_link2", link_cb);
   ASSERT(r == 0);
-  uv_run(loop);
+  uv_run(loop, UV_RUN_DEFAULT);
   ASSERT(link_cb_count == 1);
 
   r = uv_fs_open(loop, &req, "test_file_link2", O_RDWR, 0, NULL);
@@ -1179,13 +1189,14 @@ TEST_IMPL(fs_link) {
    * Run the loop just to check we don't have make any extraneous uv_ref()
    * calls. This should drop out immediately.
    */
-  uv_run(loop);
+  uv_run(loop, UV_RUN_DEFAULT);
 
   /* Cleanup. */
   unlink("test_file");
   unlink("test_file_link");
   unlink("test_file_link2");
 
+  MAKE_VALGRIND_HAPPY();
   return 0;
 }
 
@@ -1195,7 +1206,7 @@ TEST_IMPL(fs_readlink) {
 
   loop = uv_default_loop();
   ASSERT(0 == uv_fs_readlink(loop, &req, "no_such_file", dummy_cb));
-  ASSERT(0 == uv_run(loop));
+  ASSERT(0 == uv_run(loop, UV_RUN_DEFAULT));
   ASSERT(dummy_cb_count == 1);
   ASSERT(req.ptr == NULL);
   ASSERT(req.result == -1);
@@ -1208,6 +1219,7 @@ TEST_IMPL(fs_readlink) {
   ASSERT(req.errorno == UV_ENOENT);
   uv_fs_req_cleanup(&req);
 
+  MAKE_VALGRIND_HAPPY();
   return 0;
 }
 
@@ -1290,7 +1302,7 @@ TEST_IMPL(fs_symlink) {
   /* async link */
   r = uv_fs_symlink(loop, &req, "test_file", "test_file_symlink2", 0, symlink_cb);
   ASSERT(r == 0);
-  uv_run(loop);
+  uv_run(loop, UV_RUN_DEFAULT);
   ASSERT(symlink_cb_count == 1);
 
   r = uv_fs_open(loop, &req, "test_file_symlink2", O_RDWR, 0, NULL);
@@ -1313,14 +1325,14 @@ TEST_IMPL(fs_symlink) {
 
   r = uv_fs_readlink(loop, &req, "test_file_symlink2_symlink", readlink_cb);
   ASSERT(r != -1);
-  uv_run(loop);
+  uv_run(loop, UV_RUN_DEFAULT);
   ASSERT(readlink_cb_count == 1);
 
   /*
    * Run the loop just to check we don't have make any extraneous uv_ref()
    * calls. This should drop out immediately.
    */
-  uv_run(loop);
+  uv_run(loop, UV_RUN_DEFAULT);
 
   /* Cleanup. */
   unlink("test_file");
@@ -1329,6 +1341,7 @@ TEST_IMPL(fs_symlink) {
   unlink("test_file_symlink2");
   unlink("test_file_symlink2_symlink");
 
+  MAKE_VALGRIND_HAPPY();
   return 0;
 }
 
@@ -1440,6 +1453,7 @@ TEST_IMPL(fs_symlink_dir) {
   rmdir("test_dir");
   rmdir("test_dir_symlink");
 
+  MAKE_VALGRIND_HAPPY();
   return 0;
 }
 
@@ -1484,12 +1498,13 @@ TEST_IMPL(fs_utime) {
   utime_req.data = &checkme;
   r = uv_fs_utime(loop, &utime_req, path, atime, mtime, utime_cb);
   ASSERT(r == 0);
-  uv_run(loop);
+  uv_run(loop, UV_RUN_DEFAULT);
   ASSERT(utime_cb_count == 1);
 
   /* Cleanup. */
   unlink(path);
 
+  MAKE_VALGRIND_HAPPY();
   return 0;
 }
 
@@ -1521,6 +1536,7 @@ TEST_IMPL(fs_stat_root) {
   r = uv_fs_stat(loop, &stat_req, "\\\\?\\C:\\", NULL);
   ASSERT(r == 0);
 
+  MAKE_VALGRIND_HAPPY();
   return 0;
 }
 #endif
@@ -1574,12 +1590,13 @@ TEST_IMPL(fs_futime) {
   futime_req.data = &checkme;
   r = uv_fs_futime(loop, &futime_req, file, atime, mtime, futime_cb);
   ASSERT(r == 0);
-  uv_run(loop);
+  uv_run(loop, UV_RUN_DEFAULT);
   ASSERT(futime_cb_count == 1);
 
   /* Cleanup. */
   unlink(path);
 
+  MAKE_VALGRIND_HAPPY();
   return 0;
 }
 
@@ -1596,6 +1613,7 @@ TEST_IMPL(fs_stat_missing_path) {
   ASSERT(uv_last_error(loop).code == UV_ENOENT);
   uv_fs_req_cleanup(&req);
 
+  MAKE_VALGRIND_HAPPY();
   return 0;
 }
 
@@ -1621,12 +1639,13 @@ TEST_IMPL(fs_readdir_empty_dir) {
   ASSERT(r == 0);
 
   ASSERT(readdir_cb_count == 0);
-  uv_run(loop);
+  uv_run(loop, UV_RUN_DEFAULT);
   ASSERT(readdir_cb_count == 1);
 
   uv_fs_rmdir(loop, &req, path, NULL);
   uv_fs_req_cleanup(&req);
 
+  MAKE_VALGRIND_HAPPY();
   return 0;
 }
 
@@ -1647,9 +1666,10 @@ TEST_IMPL(fs_readdir_file) {
   ASSERT(r == 0);
 
   ASSERT(readdir_cb_count == 0);
-  uv_run(loop);
+  uv_run(loop, UV_RUN_DEFAULT);
   ASSERT(readdir_cb_count == 1);
 
+  MAKE_VALGRIND_HAPPY();
   return 0;
 }
 
@@ -1676,9 +1696,10 @@ TEST_IMPL(fs_open_dir) {
   ASSERT(r == 0);
 
   ASSERT(open_cb_count == 0);
-  uv_run(loop);
+  uv_run(loop, UV_RUN_DEFAULT);
   ASSERT(open_cb_count == 1);
 
+  MAKE_VALGRIND_HAPPY();
   return 0;
 }
 
@@ -1747,6 +1768,7 @@ TEST_IMPL(fs_file_open_append) {
   /* Cleanup */
   unlink("test_file");
 
+  MAKE_VALGRIND_HAPPY();
   return 0;
 }
 
@@ -1815,6 +1837,7 @@ TEST_IMPL(fs_rename_to_existing_file) {
   unlink("test_file");
   unlink("test_file2");
 
+  MAKE_VALGRIND_HAPPY();
   return 0;
 }
 
@@ -1871,5 +1894,6 @@ TEST_IMPL(fs_read_file_eof) {
   /* Cleanup */
   unlink("test_file");
 
+  MAKE_VALGRIND_HAPPY();
   return 0;
 }
