@@ -48,7 +48,7 @@ static void do_close(void* handle) {
   close_cb_called = 0;
   uv_close((uv_handle_t*)handle, close_cb);
   ASSERT(close_cb_called == 0);
-  uv_run(uv_default_loop());
+  uv_run(uv_default_loop(), UV_RUN_DEFAULT);
   ASSERT(close_cb_called == 1);
 }
 
@@ -58,6 +58,11 @@ static void fail_cb(void) {
 }
 
 
+static void fail_cb2(void) {
+  ASSERT(0 && "fail_cb2 should not have been called");
+}
+
+
 static void req_cb(uv_handle_t* req, int status) {
   req_cb_called++;
 }
@@ -95,7 +100,8 @@ static void connect_and_shutdown(uv_connect_t* req, int status) {
 
 
 TEST_IMPL(ref) {
-  uv_run(uv_default_loop());
+  uv_run(uv_default_loop(), UV_RUN_DEFAULT);
+  MAKE_VALGRIND_HAPPY();
   return 0;
 }
 
@@ -103,10 +109,11 @@ TEST_IMPL(ref) {
 TEST_IMPL(idle_ref) {
   uv_idle_t h;
   uv_idle_init(uv_default_loop(), &h);
-  uv_idle_start(&h, NULL);
+  uv_idle_start(&h, (uv_idle_cb) fail_cb2);
   uv_unref((uv_handle_t*)&h);
-  uv_run(uv_default_loop());
+  uv_run(uv_default_loop(), UV_RUN_DEFAULT);
   do_close(&h);
+  MAKE_VALGRIND_HAPPY();
   return 0;
 }
 
@@ -115,8 +122,9 @@ TEST_IMPL(async_ref) {
   uv_async_t h;
   uv_async_init(uv_default_loop(), &h, NULL);
   uv_unref((uv_handle_t*)&h);
-  uv_run(uv_default_loop());
+  uv_run(uv_default_loop(), UV_RUN_DEFAULT);
   do_close(&h);
+  MAKE_VALGRIND_HAPPY();
   return 0;
 }
 
@@ -124,10 +132,11 @@ TEST_IMPL(async_ref) {
 TEST_IMPL(prepare_ref) {
   uv_prepare_t h;
   uv_prepare_init(uv_default_loop(), &h);
-  uv_prepare_start(&h, NULL);
+  uv_prepare_start(&h, (uv_prepare_cb) fail_cb2);
   uv_unref((uv_handle_t*)&h);
-  uv_run(uv_default_loop());
+  uv_run(uv_default_loop(), UV_RUN_DEFAULT);
   do_close(&h);
+  MAKE_VALGRIND_HAPPY();
   return 0;
 }
 
@@ -135,10 +144,11 @@ TEST_IMPL(prepare_ref) {
 TEST_IMPL(check_ref) {
   uv_check_t h;
   uv_check_init(uv_default_loop(), &h);
-  uv_check_start(&h, NULL);
+  uv_check_start(&h, (uv_check_cb) fail_cb2);
   uv_unref((uv_handle_t*)&h);
-  uv_run(uv_default_loop());
+  uv_run(uv_default_loop(), UV_RUN_DEFAULT);
   do_close(&h);
+  MAKE_VALGRIND_HAPPY();
   return 0;
 }
 
@@ -154,8 +164,9 @@ TEST_IMPL(unref_in_prepare_cb) {
   uv_prepare_t h;
   uv_prepare_init(uv_default_loop(), &h);
   uv_prepare_start(&h, prepare_cb);
-  uv_run(uv_default_loop());
+  uv_run(uv_default_loop(), UV_RUN_DEFAULT);
   do_close(&h);
+  MAKE_VALGRIND_HAPPY();
   return 0;
 }
 
@@ -164,8 +175,9 @@ TEST_IMPL(timer_ref) {
   uv_timer_t h;
   uv_timer_init(uv_default_loop(), &h);
   uv_unref((uv_handle_t*)&h);
-  uv_run(uv_default_loop());
+  uv_run(uv_default_loop(), UV_RUN_DEFAULT);
   do_close(&h);
+  MAKE_VALGRIND_HAPPY();
   return 0;
 }
 
@@ -175,8 +187,9 @@ TEST_IMPL(timer_ref2) {
   uv_timer_init(uv_default_loop(), &h);
   uv_timer_start(&h, (uv_timer_cb)fail_cb, 42, 42);
   uv_unref((uv_handle_t*)&h);
-  uv_run(uv_default_loop());
+  uv_run(uv_default_loop(), UV_RUN_DEFAULT);
   do_close(&h);
+  MAKE_VALGRIND_HAPPY();
   return 0;
 }
 
@@ -185,8 +198,9 @@ TEST_IMPL(fs_event_ref) {
   uv_fs_event_t h;
   uv_fs_event_init(uv_default_loop(), &h, ".", (uv_fs_event_cb)fail_cb, 0);
   uv_unref((uv_handle_t*)&h);
-  uv_run(uv_default_loop());
+  uv_run(uv_default_loop(), UV_RUN_DEFAULT);
   do_close(&h);
+  MAKE_VALGRIND_HAPPY();
   return 0;
 }
 
@@ -196,8 +210,9 @@ TEST_IMPL(fs_poll_ref) {
   uv_fs_poll_init(uv_default_loop(), &h);
   uv_fs_poll_start(&h, NULL, ".", 999);
   uv_unref((uv_handle_t*)&h);
-  uv_run(uv_default_loop());
+  uv_run(uv_default_loop(), UV_RUN_DEFAULT);
   do_close(&h);
+  MAKE_VALGRIND_HAPPY();
   return 0;
 }
 
@@ -206,8 +221,9 @@ TEST_IMPL(tcp_ref) {
   uv_tcp_t h;
   uv_tcp_init(uv_default_loop(), &h);
   uv_unref((uv_handle_t*)&h);
-  uv_run(uv_default_loop());
+  uv_run(uv_default_loop(), UV_RUN_DEFAULT);
   do_close(&h);
+  MAKE_VALGRIND_HAPPY();
   return 0;
 }
 
@@ -217,8 +233,22 @@ TEST_IMPL(tcp_ref2) {
   uv_tcp_init(uv_default_loop(), &h);
   uv_listen((uv_stream_t*)&h, 128, (uv_connection_cb)fail_cb);
   uv_unref((uv_handle_t*)&h);
-  uv_run(uv_default_loop());
+  uv_run(uv_default_loop(), UV_RUN_DEFAULT);
   do_close(&h);
+  MAKE_VALGRIND_HAPPY();
+  return 0;
+}
+
+
+TEST_IMPL(tcp_ref2b) {
+  uv_tcp_t h;
+  uv_tcp_init(uv_default_loop(), &h);
+  uv_listen((uv_stream_t*)&h, 128, (uv_connection_cb)fail_cb);
+  uv_unref((uv_handle_t*)&h);
+  uv_close((uv_handle_t*)&h, close_cb);
+  uv_run(uv_default_loop(), UV_RUN_DEFAULT);
+  ASSERT(close_cb_called == 1);
+  MAKE_VALGRIND_HAPPY();
   return 0;
 }
 
@@ -229,10 +259,11 @@ TEST_IMPL(tcp_ref3) {
   uv_tcp_init(uv_default_loop(), &h);
   uv_tcp_connect(&connect_req, &h, addr, connect_and_shutdown);
   uv_unref((uv_handle_t*)&h);
-  uv_run(uv_default_loop());
+  uv_run(uv_default_loop(), UV_RUN_DEFAULT);
   ASSERT(connect_cb_called == 1);
   ASSERT(shutdown_cb_called == 1);
   do_close(&h);
+  MAKE_VALGRIND_HAPPY();
   return 0;
 }
 
@@ -243,11 +274,12 @@ TEST_IMPL(tcp_ref4) {
   uv_tcp_init(uv_default_loop(), &h);
   uv_tcp_connect(&connect_req, &h, addr, connect_and_write);
   uv_unref((uv_handle_t*)&h);
-  uv_run(uv_default_loop());
+  uv_run(uv_default_loop(), UV_RUN_DEFAULT);
   ASSERT(connect_cb_called == 1);
   ASSERT(write_cb_called == 1);
   ASSERT(shutdown_cb_called == 1);
   do_close(&h);
+  MAKE_VALGRIND_HAPPY();
   return 0;
 }
 
@@ -256,8 +288,9 @@ TEST_IMPL(udp_ref) {
   uv_udp_t h;
   uv_udp_init(uv_default_loop(), &h);
   uv_unref((uv_handle_t*)&h);
-  uv_run(uv_default_loop());
+  uv_run(uv_default_loop(), UV_RUN_DEFAULT);
   do_close(&h);
+  MAKE_VALGRIND_HAPPY();
   return 0;
 }
 
@@ -269,8 +302,9 @@ TEST_IMPL(udp_ref2) {
   uv_udp_bind(&h, addr, 0);
   uv_udp_recv_start(&h, (uv_alloc_cb)fail_cb, (uv_udp_recv_cb)fail_cb);
   uv_unref((uv_handle_t*)&h);
-  uv_run(uv_default_loop());
+  uv_run(uv_default_loop(), UV_RUN_DEFAULT);
   do_close(&h);
+  MAKE_VALGRIND_HAPPY();
   return 0;
 }
 
@@ -284,10 +318,11 @@ TEST_IMPL(udp_ref3) {
   uv_udp_init(uv_default_loop(), &h);
   uv_udp_send(&req, &h, &buf, 1, addr, (uv_udp_send_cb)req_cb);
   uv_unref((uv_handle_t*)&h);
-  uv_run(uv_default_loop());
+  uv_run(uv_default_loop(), UV_RUN_DEFAULT);
   ASSERT(req_cb_called == 1);
   do_close(&h);
 
+  MAKE_VALGRIND_HAPPY();
   return 0;
 }
 
@@ -296,8 +331,9 @@ TEST_IMPL(pipe_ref) {
   uv_pipe_t h;
   uv_pipe_init(uv_default_loop(), &h, 0);
   uv_unref((uv_handle_t*)&h);
-  uv_run(uv_default_loop());
+  uv_run(uv_default_loop(), UV_RUN_DEFAULT);
   do_close(&h);
+  MAKE_VALGRIND_HAPPY();
   return 0;
 }
 
@@ -307,8 +343,9 @@ TEST_IMPL(pipe_ref2) {
   uv_pipe_init(uv_default_loop(), &h, 0);
   uv_listen((uv_stream_t*)&h, 128, (uv_connection_cb)fail_cb);
   uv_unref((uv_handle_t*)&h);
-  uv_run(uv_default_loop());
+  uv_run(uv_default_loop(), UV_RUN_DEFAULT);
   do_close(&h);
+  MAKE_VALGRIND_HAPPY();
   return 0;
 }
 
@@ -318,10 +355,11 @@ TEST_IMPL(pipe_ref3) {
   uv_pipe_init(uv_default_loop(), &h, 0);
   uv_pipe_connect(&connect_req, &h, TEST_PIPENAME, connect_and_shutdown);
   uv_unref((uv_handle_t*)&h);
-  uv_run(uv_default_loop());
+  uv_run(uv_default_loop(), UV_RUN_DEFAULT);
   ASSERT(connect_cb_called == 1);
   ASSERT(shutdown_cb_called == 1);
   do_close(&h);
+  MAKE_VALGRIND_HAPPY();
   return 0;
 }
 
@@ -331,11 +369,12 @@ TEST_IMPL(pipe_ref4) {
   uv_pipe_init(uv_default_loop(), &h, 0);
   uv_pipe_connect(&connect_req, &h, TEST_PIPENAME, connect_and_write);
   uv_unref((uv_handle_t*)&h);
-  uv_run(uv_default_loop());
+  uv_run(uv_default_loop(), UV_RUN_DEFAULT);
   ASSERT(connect_cb_called == 1);
   ASSERT(write_cb_called == 1);
   ASSERT(shutdown_cb_called == 1);
   do_close(&h);
+  MAKE_VALGRIND_HAPPY();
   return 0;
 }
 
@@ -364,12 +403,13 @@ TEST_IMPL(process_ref) {
   ASSERT(r == 0);
 
   uv_unref((uv_handle_t*)&h);
-  uv_run(uv_default_loop());
+  uv_run(uv_default_loop(), UV_RUN_DEFAULT);
 
   r = uv_process_kill(&h, /* SIGTERM */ 15);
   ASSERT(r == 0);
 
   do_close(&h);
 
+  MAKE_VALGRIND_HAPPY();
   return 0;
 }
